@@ -202,6 +202,14 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
             if f is not FileFormat.lines or cls._lines_files_apply()
         }
 
+    @classmethod
+    def from_records(
+        cls,
+        *args,
+        **kwargs,
+    ) -> __qualname__:
+        return cls.convert(super().from_records(*args, **kwargs))
+
     def to_lines(
         self,
         path_or_buff,
