@@ -3,13 +3,29 @@ Tools that could possibly be used outside of typed-dfs.
 """
 from __future__ import annotations
 
+import collections
+import functools
 import hashlib
 import os
 import re
 import sys
 from pathlib import Path, PurePath
-from typing import Optional, Sequence, Mapping, Set, Union
+from typing import (
+    Optional,
+    Sequence,
+    Mapping,
+    Set,
+    Union,
+    Any,
+    ValuesView,
+    TypeVar,
+    List,
+    Iterator,
+    AbstractSet,
+    Dict,
+)
 
+import numpy as np
 from pandas.io.common import get_handle
 
 # noinspection PyProtectedMember
@@ -29,9 +45,222 @@ from typeddfs.df_errors import (
 
 _hex_pattern = re.compile(r"[A-Ha-h0-9]+")
 _hashsum_file_sep = re.compile(r" [ *]")
+T = TypeVar("T", covariant=True)
+K = TypeVar("K", contravariant=True)
+V = TypeVar("V", covariant=True)
+
+
+@functools.total_ordering
+class FrozeList(Sequence[T]):
+    """
+    An immutable list.
+    Hashable and ordered.
+    """
+
+    def __init__(self, lst: Sequence[T]):
+        self.__lst = lst if isinstance(lst, list) else list(lst)
+        try:
+            self.__hash = hash(tuple(lst))
+        except AttributeError:
+            self.__hash = 0
+
+    def __iter__(self) -> Iterator[T]:
+        return iter(self.__lst)
+
+    def __getitem__(self, item: int):
+        return self.__lst[item]
+
+    def __hash__(self) -> int:
+        return self.__hash
+
+    def __eq__(self, other: Union[FrozeList[T], Sequence[T]]) -> bool:
+        return self.__lst == self.__make_other(other)
+
+    def __lt__(self, other: Union[FrozeList[T], Sequence[T]]):
+        return self.__lst < self.__make_other(other)
+
+    def __len__(self) -> int:
+        return len(self.__lst)
+
+    def __str__(self) -> str:
+        return str(self.__lst)
+
+    def __repr__(self) -> str:
+        return repr(self.__lst)
+
+    def to_list(self) -> List[T]:
+        return list(self.__lst)
+
+    def __make_other(self, other: Union[FrozeList[T], Sequence[T]]) -> List[T]:
+        if isinstance(other, FrozeList):
+            other = other.__lst
+        if isinstance(other, list):
+            return other
+        elif isinstance(other, Sequence):
+            return list(other)
+        raise TypeError(f"Cannot compare to {type(other)}")
+
+
+class FrozeSet(AbstractSet[T]):
+    """
+    An immutable set.
+    Hashable.
+    This is almost identical to ``typing.FrozenSet``, but it's behavior was made
+    equivalent to those of FrozeDict and FrozeList.
+    """
+
+    def __init__(self, lst: AbstractSet[T]):
+        self.__lst = lst if isinstance(lst, set) else set(lst)
+        try:
+            self.__hash = hash(tuple(lst))
+        except AttributeError:
+            self.__hash = 0
+
+    def get(self, item: T, default: Optional[T]) -> Optional[T]:
+        if item in self.__lst:
+            return item
+        return default
+
+    def __getitem__(self, item: T) -> T:
+        if item in self.__lst:
+            return item
+        raise KeyError(f"Item {item} not found")
+
+    def __contains__(self, x: T) -> bool:
+        return x in self.__lst
+
+    def __iter__(self) -> Iterator[T]:
+        return iter(self.__lst)
+
+    def __hash__(self) -> int:
+        return self.__hash
+
+    def __eq__(self, other: FrozeSet[T]) -> bool:
+        return self.__lst == self.__make_other(other)
+
+    def __len__(self) -> int:
+        return len(self.__lst)
+
+    def __str__(self) -> str:
+        return str(self.__lst)
+
+    def __repr__(self) -> str:
+        return repr(self.__lst)
+
+    def to_set(self) -> AbstractSet[T]:
+        return set(self.__lst)
+
+    def __make_other(self, other: Union[FrozeSet[T], AbstractSet[T]]) -> Set[T]:
+        if isinstance(other, FrozeSet):
+            other = other.__lst
+        if isinstance(other, set):
+            return other
+        elif isinstance(other, AbstractSet):
+            return set(other)
+        raise TypeError(f"Cannot compare to {type(other)}")
+
+
+class FrozeDict(Mapping[K, V]):
+    """
+    An immutable dictionary/mapping.
+    Hashable.
+    """
+
+    def __init__(self, dct: Mapping[K, V]):
+        self.__dct = dct if isinstance(dct, dict) else dict(dct)
+        self.__hash = hash(tuple(dct.items()))
+
+    def __iter__(self):
+        return iter(self.__dct)
+
+    def get(self, key: K, default: Optional[V] = None) -> Optional[V]:
+        return self.__dct.get(key)
+
+    def req(self, key: K, default: Optional[V] = None) -> V:
+        if default is None:
+            return self.__dct[key]
+        return self.__dct.get(key, default)
+
+    def items(self) -> AbstractSet[tuple[K, V]]:
+        return self.__dct.items()
+
+    def keys(self) -> AbstractSet[K]:
+        return self.__dct.keys()
+
+    def values(self) -> ValuesView[V]:
+        return self.__dct.values()
+
+    def __getitem__(self, item: K) -> T:
+        return self.__dct[item]
+
+    def __hash__(self) -> int:
+        return self.__hash
+
+    def __eq__(self, other: FrozeDict[K, V]) -> bool:
+        if isinstance(self, FrozeDict):
+            return self.__dct == other.__dct
+        elif isinstance(self, dict):
+            return self == other.__dct
+        elif isinstance(self, Mapping):
+            return self == dict(other.__dct)
+        raise TypeError(f"Cannot compare to {type(other)}")
+
+    def __len__(self) -> int:
+        return len(self.__dct)
+
+    def to_dict(self) -> Mapping[K, V]:
+        return dict(self.__dct)
+
+    def __make_other(self, other: Union[FrozeDict[K, V], Mapping[K, V]]) -> Dict[K, V]:
+        if isinstance(other, FrozeDict):
+            other = other.__dct
+        if isinstance(other, dict):
+            return other
+        elif isinstance(other, Mapping):
+            return dict(other)
+        raise TypeError(f"Cannot compare to {type(other)}")
 
 
 class Utils:
+    @classmethod
+    def freeze(cls, v: Any) -> Any:
+        """
+        Returns ``v`` or a hashable view of it.
+        Note that the returned types must be hashable but might not be ordered.
+        You can generally add these values as DataFrame elements, but you might not
+        be able to sort on those columns.
+
+        Args:
+            v: Any value
+
+        Returns:
+            Either ``v`` itself,
+            a :py.type:`typeddfs.utils.FrozeSet` (subclass of :py.type:`typing.AbstractSet`),
+            a :py.type:`typeddfs.utils.FrozeList` (subclass of :py.type:`typing.Sequence`),
+            or a :py.type:`typeddfs.utils.FrozeDict` (subclass of :py.type:`typing.Mapping`).
+            int, float, str, np.generic, and tuple are always returned as-is.
+
+        Raises:
+            AttributeError: If ``v`` is not hashable and could not converted to
+                            a FrozeSet, FrozeList, or FrozeDict, *or* if one of the elements for
+                            one of the above types is not hashable.
+            TypeError: If ``v`` is a :py.type:`iterator.Iterator` or :py.type:`collections.deque`
+        """
+        if isinstance(v, (int, float, str, np.generic, tuple, frozenset)):
+            return v  # short-circuit
+        if isinstance(v, Iterator):  # let's not ruin their iterator by traversing
+            raise TypeError("Type is an iterator")
+        if isinstance(v, collections.deque):  # the only other major built-in type we won't accept
+            raise TypeError("Type is a deque")
+        if isinstance(v, Sequence):
+            return FrozeList(v)
+        if isinstance(v, AbstractSet):
+            return frozenset(v)
+        if isinstance(v, Mapping):
+            return FrozeDict(v)
+        hash(v)  # let it raise an AttributeError
+        return v
+
     @classmethod
     def default_hash_algorithm(cls) -> str:
         return _DEFAULT_HASH_ALG
@@ -440,4 +669,4 @@ class Utils:
             )
 
 
-__all__ = ["Utils", "TableFormat"]
+__all__ = ["Utils", "TableFormat", "FrozeList", "FrozeDict", "FrozeSet"]
